@@ -5,6 +5,9 @@ import os
 from execute import execWithCapture, execWithCaptureErrorStatus, execWithCaptureStatus, execWithCaptureProgress, execWithCaptureErrorStatusProgress, execWithCaptureStatusProgress
 from CommandError import *
 
+from Cluster import Cluster
+
+
 import xml
 import xml.dom
 from xml.dom import minidom
@@ -22,7 +25,7 @@ FSCREATE_FAILURE=_("Creation of filesystem failed. Command attempted: \"%s\" - S
 FSRESIZE_FAILURE=_("Resize of filesystem failed. Command attempted: \"%s\" - System Error Message: %s")
 FSCHECK_FAILURE=_("Check of filesystem failed. Command attempted: \"%s\" - System Error Message: %s")
 FSUPGRADE_FAILURE=_("Upgrade of filesystem failed. Command attempted: \"%s\" - System Error Message: %s")
-
+gfs_types = {"1309":"gfs","1801":"gfs2"}
 
 
 def get_fs(path):
@@ -43,8 +46,8 @@ def get_fs(path):
         return Unknown('xfs')
     elif re.search('jfs', result, re.I):
         return Unknown('jfs')
-    elif re.search('raiserfs', result, re.I):
-        return Unknown('raiserfs')
+    elif re.search('reiserfs', result, re.I):
+        return Unknown('reiserfs')
     elif re.search('swap', result, re.I):
         return Unknown('swap')
     else:
@@ -148,7 +151,7 @@ class ext3(Filesystem):
         creatable = self.check_path('/sbin/mkfs.ext3')
         mountable = self.check_mountable('ext3', 'ext3')
         resize_offline = self.check_paths(['/sbin/e2fsck', '/sbin/resize2fs'])
-        extend_online = self.check_path('/usr/sbin/ext2online')
+        extend_online, dummy = self.__extend_online_cmd()
         
         Filesystem.__init__(self, 'Ext3', creatable, True, mountable, 
                             extend_online, resize_offline, False, resize_offline,
@@ -173,14 +176,27 @@ class ext3(Filesystem):
             raise CommandError('FATAL', FSCREATE_FAILURE % (cmdstr,e))
     
     def extend_online(self, dev_path):
-        args = list()
-        args.append('/usr/sbin/ext2online')
-        args.append(dev_path)
+        dummy, cmd = self.__extend_online_cmd()
+        args = [cmd, dev_path]
         cmdstr = ' '.join(args)
         msg = RESIZING_FS % (self.name)
-        o,e,r = execWithCaptureErrorStatusProgress('/usr/sbin/ext2online', args, msg)
-        if r != 0:
-            raise CommandError('FATAL', FSRESIZE_FAILURE % (cmdstr,e))
+        o, e, s = execWithCaptureErrorStatusProgress(cmd, args, msg)
+        if s != 0:
+            raise CommandError('FATAL', FSRESIZE_FAILURE % (cmdstr, e))
+    def __extend_online_cmd(self):
+        # if ext2online is present, use it
+        cmd = '/usr/sbin/ext2online'
+        if self.check_path(cmd):
+            return (True, cmd)
+        # resize2fs 1.39 and above are also able to online extend ext3
+        try:
+            cmd = '/sbin/resize2fs'
+            o, e, s = execWithCaptureErrorStatus(cmd, [cmd])
+            if (s == 0 or s == 1) and float(e.strip().split()[1]) >= 1.39:
+                return (True, cmd)
+        except:
+            pass
+        return (False, None)
     
     def reduce_online(self, dev_path, new_size_bytes):
         # not supported
@@ -376,8 +392,11 @@ class gfs(Filesystem):
     def probe(self, path):
         l_type = self.__get_gfs_lock_type(path)
         if l_type == 'nolock':
-            return True
+            gfs_type = self.__get_gfs_fstype(path)
+            if gfs_type == 'gfs':
+                return True
         return False
+
     
     def create(self, path):
         MKFS_GFS_BIN='/sbin/gfs_mkfs'
@@ -420,9 +439,23 @@ class gfs(Filesystem):
                     return 'dlm'
                 elif 'lock_gulm' in o:
                     return 'gulm'
-                elif 'lock_nolock' in o:
+                elif 'nolock' in o:
                     return 'nolock'
         return None
+
+    def __get_gfs_fstype(self, path):
+        if self.check_path('/sbin/gfs_tool'):
+            args = ['/sbin/gfs_tool']
+            args.append('sb')
+            args.append(path)
+            args.append('ondisk')
+            cmdstr = ' '.join(args)
+            o,e,r = execWithCaptureErrorStatus('/sbin/gfs_tool', args)
+            if r == 0:
+                for k,v in gfs_types.iteritems():
+                    if k in o: 
+                        return v
+        return None
     
 
 
@@ -479,9 +512,11 @@ class gfs_clustered(Filesystem):
                 c_running = self.__is_cluster_running(c_name)
                 gfs_clustername = self.__get_gfs_table_name(path)[0]
                 self.mountable = (gfs_clustername == c_name) and c_running and (gfs_lock == c_lock)
-            return True
+            gfs_type = self.__get_gfs_fstype(path)
+            if gfs_type == "gfs":
+                return True
         return False
-    
+ 
     def create(self, path):
         if not self.creatable:
             raise "not creatable"
@@ -566,7 +601,7 @@ class gfs_clustered(Filesystem):
                     return 'dlm'
                 elif 'lock_gulm' in o:
                     return 'gulm'
-                elif 'lock_nolock' in o:
+                elif 'nolock' in o:
                     return 'nolock'
         return None
     def __get_gfs_table_name(self, path):
@@ -583,48 +618,16 @@ class gfs_clustered(Filesystem):
                     return words[5].strip('\"').split(':')
         return (None, None)
     
-    
     def __get_cluster_name(self):
-        return self.__get_cluster_info()[0]
+        return Cluster().get_name()
     def __get_cluster_lock_type(self):
-        return self.__get_cluster_info()[1]
+        return Cluster().get_lock_type()
     def __get_cluster_nodes_num(self):
-        return self.__get_cluster_info()[2]
-    def __get_cluster_info(self):
-        try:
-            c_conf = minidom.parseString(file('/etc/cluster/cluster.conf').read(10000000)).firstChild
-            name = c_conf.getAttribute('name')
-            lock = None
-            nodes_num = 0
-            for node in c_conf.childNodes:
-                if node.nodeType == xml.dom.Node.ELEMENT_NODE:
-                    if node.nodeName == 'cman':
-                        lock = 'dlm'
-                    elif node.nodeName == 'gulm':
-                        lock = 'gulm'
-                    elif node.nodeName == 'clusternodes':
-                        nodes = node
-                        for node in nodes.childNodes:
-                            if node.nodeType == xml.dom.Node.ELEMENT_NODE:
-                                if node.nodeName == 'clusternode':
-                                    nodes_num += 1
-            if lock != None:
-                return (name, lock, nodes_num)
-        except:
-            pass
-        return (None, None, None)
+        return Cluster().get_nodes_num()
     def __is_cluster_running(self, clustername):
         if clustername == None:
             return False
-        try:
-            args = ['/sbin/magma_tool', 'quorum']
-            o, e, s = execWithCaptureErrorStatus('/sbin/magma_tool', args)
-            if s == 0:
-                if o.find('Quorate') != -1:
-                    return True
-        except:
-            pass
-        return False
+        return Cluster().running()
     
     def __errorMessage(self, message):
         dlg = gtk.MessageDialog(None, 0,
@@ -635,6 +638,20 @@ class gfs_clustered(Filesystem):
         dlg.destroy()
         return rc
 
+    def __get_gfs_fstype(self, path):
+        if self.check_path('/sbin/gfs_tool'):
+            args = ['/sbin/gfs_tool']
+            args.append('sb')
+            args.append(path)
+            args.append('ondisk')
+            cmdstr = ' '.join(args)
+            o,e,r = execWithCaptureErrorStatus('/sbin/gfs_tool', args)
+            if r == 0:
+                for k,v in gfs_types.iteritems():
+                    if k in o:
+                        return v
+        return None
+
     
 
 
@@ -652,7 +669,9 @@ class gfs2(Filesystem):
     def probe(self, path):
         l_type = self.__get_gfs_lock_type(path)
         if l_type == 'nolock':
-            return True
+            gfs_type = self.__get_gfs_fstype(path)
+            if gfs_type == "gfs2":
+                return True
         return False
     
     def create(self, path):
@@ -696,10 +715,24 @@ class gfs2(Filesystem):
                     return 'dlm'
                 elif 'lock_gulm' in o:
                     return 'gulm'
-                elif 'lock_nolock' in o:
+                elif 'nolock' in o:
                     return 'nolock'
         return None
     
+    def __get_gfs_fstype(self, path):
+        if self.check_path('/sbin/gfs_tool'):
+            args = ['/sbin/gfs_tool']
+            args.append('sb')
+            args.append(path)
+            args.append('ondisk')
+            cmdstr = ' '.join(args)
+            o,e,r = execWithCaptureErrorStatus('/sbin/gfs_tool', args)
+            if r == 0:
+                for k,v in gfs_types.iteritems():
+                    if k in o: 
+                        return v
+        return None
+
 
 
 class gfs2_clustered(Filesystem):
@@ -755,7 +788,9 @@ class gfs2_clustered(Filesystem):
                 c_running = self.__is_cluster_running(c_name)
                 gfs_clustername = self.__get_gfs_table_name(path)[0]
                 self.mountable = (gfs_clustername == c_name) and c_running and (gfs_lock == c_lock)
-            return True
+            gfs_type = self.__get_gfs_fstype(path)
+            if gfs_type == "gfs2":
+                return True
         return False
     
     def create(self, path):
@@ -842,7 +877,7 @@ class gfs2_clustered(Filesystem):
                     return 'dlm'
                 elif 'lock_gulm' in o:
                     return 'gulm'
-                elif 'lock_nolock' in o:
+                elif 'nolock' in o:
                     return 'nolock'
         return None
     def __get_gfs_table_name(self, path):
@@ -859,48 +894,16 @@ class gfs2_clustered(Filesystem):
                     return words[5].strip('\"').split(':')
         return (None, None)
     
-    
     def __get_cluster_name(self):
-        return self.__get_cluster_info()[0]
+        return Cluster().get_name()
     def __get_cluster_lock_type(self):
-        return self.__get_cluster_info()[1]
+        return Cluster().get_lock_type()
     def __get_cluster_nodes_num(self):
-        return self.__get_cluster_info()[2]
-    def __get_cluster_info(self):
-        try:
-            c_conf = minidom.parseString(file('/etc/cluster/cluster.conf').read(10000000)).firstChild
-            name = c_conf.getAttribute('name')
-            lock = None
-            nodes_num = 0
-            for node in c_conf.childNodes:
-                if node.nodeType == xml.dom.Node.ELEMENT_NODE:
-                    if node.nodeName == 'cman':
-                        lock = 'dlm'
-                    elif node.nodeName == 'gulm':
-                        lock = 'gulm'
-                    elif node.nodeName == 'clusternodes':
-                        nodes = node
-                        for node in nodes.childNodes:
-                            if node.nodeType == xml.dom.Node.ELEMENT_NODE:
-                                if node.nodeName == 'clusternode':
-                                    nodes_num += 1
-            if lock != None:
-                return (name, lock, nodes_num)
-        except:
-            pass
-        return (None, None, None)
+        return Cluster().get_nodes_num()
     def __is_cluster_running(self, clustername):
         if clustername == None:
             return False
-        try:
-            args = ['/sbin/magma_tool', 'quorum']
-            o, e, s = execWithCaptureErrorStatus('/sbin/magma_tool', args)
-            if s == 0:
-                if o.find('Quorate') != -1:
-                    return True
-        except:
-            pass
-        return False
+        return Cluster().running()
     
     def __errorMessage(self, message):
         dlg = gtk.MessageDialog(None, 0,
@@ -911,4 +914,17 @@ class gfs2_clustered(Filesystem):
         dlg.destroy()
         return rc
 
-    
+    def __get_gfs_fstype(self, path):
+        if self.check_path('/sbin/gfs_tool'):
+            args = ['/sbin/gfs_tool']
+            args.append('sb')
+            args.append(path)
+            args.append('ondisk')
+            cmdstr = ' '.join(args)
+            o,e,r = execWithCaptureErrorStatus('/sbin/gfs_tool', args)
+            if r == 0:
+                for k,v in gfs_types.iteritems():
+                    if k in o:
+                        return v
+        return None
+
