@@ -1,5 +1,3 @@
-#!/usr/bin/python
-
 import sys
 import types
 import select
@@ -17,6 +15,8 @@ from lvmui_constants import *
 from WaitMsg import WaitMsg
 from execute import ForkedCommand, execWithCapture
 
+
+
 import stat
 import gettext
 _ = gettext.gettext
@@ -52,30 +52,6 @@ class Volume_Tab_View:
                                                                                 
     self.model_factory = model_factory
     
-    # check locking type
-    locking_type = self.model_factory.get_locking_type()
-    if locking_type != 1:
-        should_exit = False
-        if locking_type == 0:
-            msg = _("LVM locks are disabled!!! \nMassive data corruption may occur.\nEnable locking (locking_type=1 or 2 in /etc/lvm/lvm.conf).")
-            should_exit = True
-        elif locking_type == 2:
-            ps_out = execWithCapture('/bin/ps', ['/bin/ps', '-A'])
-            if ps_out.find('clvmd') == -1:
-                msg = _("LVM is configured to use Cluster Locking mechanism, but clvmd daemon is not running. Start daemon with command:\nservice clvmd start \nor, turn off cluster locking (locking_type=1 in /etc/lvm/lvm.conf).")
-                should_exit = True
-        else:
-            msg = _("%s only supports file and cluster based locking (locking_type=1 or 2 in /etc/lvm/lvm.conf).")
-            msg = msg % PROGNAME
-            should_exit = True
-        if should_exit:
-            dlg = gtk.MessageDialog(None, 0,
-                                    gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
-                                    msg)
-            dlg.run()
-            sys.exit(10)
-    
-    
     self.main_win = app
     self.width = 0
     self.height = 0
