@@ -128,17 +128,20 @@ PV_UUID_IDX = 8
 
 
 LVS_HAS_ALL_OPTION = False
-out, status = execWithCaptureStatus(LVM_BIN_PATH, [LVM_BIN_PATH, 'lvs', '--all'])
-if status == 0:
-  LVS_HAS_ALL_OPTION = True
-
 LVS_HAS_MIRROR_OPTIONS = False
-LVS_HAS_MIRROR_OPTIONS = LVS_HAS_ALL_OPTION
 
 
 class lvm_model:
   
   def __init__(self):
+    global LVS_HAS_ALL_OPTION
+    global LVS_HAS_MIRROR_OPTIONS
+    
+    out, status = execWithCaptureStatus(LVM_BIN_PATH, [LVM_BIN_PATH, 'lvs', '--all'])
+    if status == 0:
+      LVS_HAS_ALL_OPTION = True
+    LVS_HAS_MIRROR_OPTIONS = LVS_HAS_ALL_OPTION
+    
     self.__block_device_model = BlockDeviceModel()
     self.__VGs = {}
     self.__PVs = []
@@ -147,6 +150,11 @@ class lvm_model:
     progress = ProgressPopup(RELOAD_LVM_MESSAGE)
     progress.start()
     
+    # clean-up everything
+    self.__block_device_model = BlockDeviceModel()
+    self.__VGs = {}
+    self.__PVs = []
+    
     # first query VolumeGroups
     self.__VGs = {}
     for vg in self.__query_VGs():
@@ -488,7 +496,7 @@ class lvm_model:
       
       segment = None
       devs = devices.split(',')
-      if attrs[0] == 'm':
+      if attrs[0] == 'm' or attrs[0] == 'M':
         # mirrored LV
         lv.set_mirror_log(mirror_log) # tmp, will get replaced with real one at __link_mirrors()
         segment = MirroredSegment(seg_start, seg_size)
@@ -943,6 +951,23 @@ class lvm_model:
     for prop in end:
       text_list.append(prop)
     
+    try:
+      # add scsi dev info
+      device = pv.getDevnames()[0]
+      devname = pv.extract_name(device)
+      SCSI_ID_BIN = '/sbin/scsi_id'
+      args = [SCSI_ID_BIN, '-g', '-i', '-u', '-s', '/block/' + devname]
+      o, e, s = execWithCaptureErrorStatus(SCSI_ID_BIN, args)
+      if s == 0:
+        scsi_addr, scsi_id = o.strip().split()
+        scsi_addr = scsi_addr.strip(':')
+        text_list.append(_("SCSI Address:  "))
+        text_list.append(scsi_addr)
+        text_list.append(_("SCSI ID:  "))
+        text_list.append(scsi_id)
+    except:
+      pass
+    
     return text_list
   
   def __getFS(self, path):
@@ -993,17 +1018,19 @@ class lvm_model:
   
   def is_mirroring_supported(self):
     return LVS_HAS_MIRROR_OPTIONS
+
+
   
-  def get_locking_type(self):
-    conf = open('/etc/lvm/lvm.conf')
-    
-    lines = conf.readlines()
-    for line in lines:
-      words = line.split()
-      if len(words) < 3:
-        continue
-      if words[0] == 'locking_type':
-        if words[1] == '=':
-          locking_type = int(words[2])
-          return locking_type
-    return None
+def lvm_conf_get_locking_type():
+  conf = open('/etc/lvm/lvm.conf')
+  
+  lines = conf.readlines()
+  for line in lines:
+    words = line.split()
+    if len(words) < 3:
+      continue
+    if words[0] == 'locking_type':
+      if words[1] == '=':
+        locking_type = int(words[2])
+        return locking_type
+  return None
